import os
import sys

ROOT = os.path.dirname(os.path.dirname(__file__))

os.umask(0o022)


def jroot(*parts: str) -> str:
    return os.path.join(ROOT, *parts)


# Default settings
from .default import *

# docker settings
if os.path.exists(jroot('config', 'docker.py')):
    from .docker import *

# Local settings
if os.path.exists(jroot('config', 'local.py')):
    from .local import *

# Package settings
if os.path.exists('/etc/eportal/config'):
    import runpy

    ctx = runpy.run_path('/etc/eportal/config')
    globals().update((k, v) for k, v in ctx.items() if not k.startswith('__'))

# override setting with ENV var, usefull for tests
thismodule = sys.modules[__name__]
for varname in locals().copy():
    env_varname = f'EPORTAL_TEST_SETTINGS_{varname}'
    if env_varname in os.environ:
        setattr(thismodule, varname, os.environ[env_varname])

if not os.path.exists(BASE_DIR):
    os.makedirs(BASE_DIR, exist_ok=True)

if not PATCHES_DIR:
    PATCHES_DIR = BASE_DIR

if not PATCH_RESOURCES_DIR:
    PATCH_RESOURCES_DIR = os.path.join(PATCHES_DIR, 'resources')

if not RESOURCES_OVERRIDE_DIR:
    RESOURCES_OVERRIDE_DIR = os.path.join(BASE_DIR, 'resources_override')

if not USER_PATCH_RESOURCES_DIR:
    USER_PATCH_RESOURCES_DIR = os.path.join(PATCHES_DIR, 'user-resources')

if not ARCH_DIR:
    ARCH_DIR = os.path.join(PATCHES_DIR, 'arch')

if not DATABASE_URI:
    DATABASE_URI = f'sqlite:///{BASE_DIR}/data.sqlite'

if not SPOOLER_DATABASE_URI:
    SPOOLER_DATABASE_URI = f'sqlite:///{BASE_DIR}/spooler.sqlite'

if not PATCHES_DATA_DATABASE_URI:
    PATCHES_DATA_DATABASE_URI = f'sqlite:///{BASE_DIR}/patches-data.sqlite'

if not OPLOG_DATABASE:
    OPLOG_DATABASE = f'{BASE_DIR}/oplog.sqlite'

# legacy databases
if not PATCHES_DATABASE_URI:
    PATCHES_DATABASE_URI = f'sqlite:///{BASE_DIR}/patches.sqlite'

if not USER_PATCHES_DATABASE_URI:
    USER_PATCHES_DATABASE_URI = f'sqlite:///{BASE_DIR}/user-patches.sqlite'

if not QEMU_PATCHES_DATABASE_URI:
    QEMU_PATCHES_DATABASE_URI = f'sqlite:///{BASE_DIR}/qemu-patches.sqlite'

if not DB_PATCHES_DATABASE_URI:
    DB_PATCHES_DATABASE_URI = f'sqlite:///{BASE_DIR}/db-patches.sqlite'
# end of legacy databases

if AUTH_SESSION_LIFETIME:
    PERMANENT_SESSION_LIFETIME: Optional[int] = AUTH_SESSION_LIFETIME

# Use relaxed cookie policy to allow cross site redirects during OAuth2 flow
if OIDC_CLIENT_ID and SESSION_COOKIE_SAMESITE == 'Strict':
    SESSION_COOKIE_SAMESITE = 'Lax'

if LOGGING:
    import logging.config

    logging.config.dictConfig(LOGGING)
else:
    logging.basicConfig(
        level=LOGGING_LEVEL,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    )
