import logging
import os
from typing import Any, Dict, List, Optional, Union

SECRET_KEY: bytes = os.environb.get(b'SECRET_KEY') or os.urandom(32)
SESSION_COOKIE_SAMESITE = 'Strict'

BASE_DIR = '/var/lib/eportal'

_LEGACY_BASE_DIR = '/usr/share/kcare-eportal'
if os.path.isfile(os.path.join(_LEGACY_BASE_DIR, 'data.sqlite')):
    BASE_DIR = _LEGACY_BASE_DIR

PATCHES_DIR = ''  # default is BASE_DIR

# Databases are located in BASE_DIR/*.sqlite by default
DATABASE_URI = ''
SPOOLER_DATABASE_URI = ''
PATCHES_DATA_DATABASE_URI = ''
OPLOG_DATABASE = ''

# legacy databases
PATCHES_DATABASE_URI = ''
USER_PATCHES_DATABASE_URI = ''
QEMU_PATCHES_DATABASE_URI = ''
DB_PATCHES_DATABASE_URI = ''

PATCH_RESOURCES_DIR = ''  # PATCHES_DIR + '/resources'
USER_PATCH_RESOURCES_DIR = ''  # PATCHES_DIR + '/user-resources'
ARCH_DIR = ''  # PATCHES_DIR + '/arch'
RESOURCES_OVERRIDE_DIR = ''  # PATCHES_DIR + '/resources_override'

LOGGING: Optional[Dict[str, Any]] = None
LOGGING_LEVEL: Union[str, int] = logging.INFO

DATETIME_FORMAT = '%m/%d/%y %H:%M:%S'
USE_LOCAL_TIMEZONE_FOR_API: bool = False

ECHO_SQL = False

CLEAN_INACTIVE_SERVERS_PERIOD: Optional[int] = None
SERVER_STATS_PERIOD = 3600 * 24  # every day
AUTO_UPDATE_PERIOD: Optional[int] = 600  # check update necessity each 10 minutes
RECONCILE_PATCHES_PERIOD = 600  # reconcile patches each 10 minutes

REVERSE_PATCHSETS = True  # disable for testing purposes
TEST_SEND_FILE = False  # return resolved file paths

PROXY = ''  # url to http/socks4/socks5 proxy
CA_BUNDLE = ''  # one can specify custom path to ssl certificates
USE_CERTIFI = False  # by default, eportal will use system certificates if available

REDUCED_REPORT = False

CLIENT_CERTIFICATE = ''
LIMIT_PATCHSETS: Optional[int] = None

OIDC_AUTH_URL = ''
# A false positive based on TOKEN in the name, suppress Bandit warning.
OIDC_TOKEN_URL = ''  # nosec B105
OIDC_USERINFO_URL = ''
OIDC_LOGOUT_URL = ''
OIDC_CLIENT_ID = ''
# A false positive based on SECRET in the name, suppress Bandit warning.
OIDC_CLIENT_SECRET = ''  # nosec B105

# replication
NODE_URL = ''
LEADER_URL = ''
REPLICATION_SHARED_KEY = ''
REPLICATION_CHECK_CERT = True
REPLICATION_CLEAN_AGE = 86400 * 7  # 7 days, data age in seconds to leave during oplog clean
REPLICATION_CLEAN_AGGRESSIVE_AGE = 3600  # 4 hours, data age in seconds to leave during oplog aggressive clean
REPLICATION_STANDBY = False  # in standby mode node only could receive changes

# auth
AUTH_RESTRICTED_INDEX = True
AUTH_PASSWORD_MIN_LENGTH = 10  # min password lenght
AUTH_SESSION_LIFETIME: Optional[int] = None  # seconds, by default session will last till browser closing


# AUTH_REFRESH_SESSION
# if False, expires session after lifetime seconds after login
# if True, expires session after lifetime seconds of inactivity
AUTH_REFRESH_SESSION = False

# CACHE_MODE
# Enable cache mode. It allows to pass patch request directly to patchserver without downloading
# full patchset archive.
CACHE_MODE = False

PATCH_SOURCE_URL = 'https://downloads.kernelcare.com/patch-download/'
PATCH_SOURCE_USER = ''
# A false positive based on PASSWORD in the name, suppress Bandit warning.
PATCH_SOURCE_PASSWORD = ''  # nosec B105

DEFAULT_FS_OWNER = "eportal:eportal"
DEFAULT_NGINX_CACHE_OWNER = "www-data:www-data"

WEBSEAL_AUTH = False
WEBSEAL_AUTH_HEADER = ''
WEBSEAL_AUTH_DEFAULT_ROLE = 'readonly'

PUSH_METRICS = False

PATCHSERVER_URL = ''

ALLOWED_HOSTS: List[str] = []
UI_DEBUG = False

COUNT_CHECKIN_PROCESSING = True

NGINX_RESOLVER = '8.8.8.8 valid=300s'

NGINX_CFG_PATH = '/etc/nginx/conf.d/eportal-private.conf'

REBOOTS_QUEUE_TIMEOUT = 60.0
REBOOTS_QUEUE_MAX_BATCH_SIZE = 200
REBOOTS_EXTENDED_STATS = False

BUNITS = False
BUNITS_LOGIN_SELECTOR = False
BUNITS_SELECTOR = False

CHECKIN_QUEUE_TIMEOUT = 5

REGISTRATION_TOKEN_INCLUDES_KEY = False
RESTORE_REGISTRATIONS_FROM_TOKEN = False
RESTORE_REGISTRATIONS_MIN_TOKEN_AGE = 3600
