from datetime import datetime
from typing import Optional

from sqlalchemy import Column, Integer, Text, DateTime, Index
from sqlalchemy.orm.decl_api import Mapped, mapped_column

from eportal.db import Base, session
from eportal.dbutils import transaction

ACTION_LOGIN = 'login'
ACTION_LOGOUT = 'logout'
ACTION_MANAGE_PATCHSET = 'manage-patchset'
ACTION_MANAGE_KEY = 'manage-key'
ACTION_MANAGE_FEED = 'manage-feed'
ACTION_MANAGE_USER = 'manage-user'
ACTION_MANAGE_UNIT = 'manage-unit'
ACTION_MANAGE_PATCH_SOURCE = 'manage-patch-source'
ACTION_MANAGE_LDAP = 'manage-ldap'
ACTION_UPDATE_SIGNATURE = 'update-signature'
ACTION_IMPORT_DATA = 'import-data'


class AuditLog(Base):
    __tablename__ = 'auditlog'

    id = Column(Integer, primary_key=True)
    unit_id: Mapped[int] = mapped_column(Integer, nullable=False, default=0, server_default='0')
    date: Mapped[datetime] = mapped_column(DateTime, nullable=False, index=True)
    user: Mapped[str] = mapped_column(Text(), nullable=False)
    action: Mapped[str] = mapped_column(Text(), nullable=False)
    details: Mapped[str] = mapped_column(Text(), nullable=False)


Index('idx_auditlog_user_date', AuditLog.user, AuditLog.date.desc())
Index('idx_auditlog_action_date', AuditLog.action, AuditLog.date.desc())


@transaction(session)
def add_record(unit_id: int, user: str, action: str, details: str, now: Optional[datetime] = None) -> None:
    now = now or datetime.utcnow()
    obj = AuditLog(unit_id=unit_id, date=now, user=user, action=action, details=details)
    session.add(obj)
