import sys

from flask.app import Flask

from eportal.web import app


def print_routes(app: Flask, sort: bool = False) -> None:
    """Print all project routes"""
    rules = list(app.url_map.iter_rules())
    if sort:
        rules = sorted(rules, key=lambda r: r.rule)

    for rule in rules:
        if rule.methods is None:
            raise NotImplementedError('Default Rule.methods expected to be assigned by Flask')
        func = app.view_functions[rule.endpoint]
        print(
            '{0:10} {1} {2}\t{3}.{4}'.format(
                ','.join(rule.methods),
                rule.rule,
                rule.endpoint,
                func.__module__,
                func.__name__,
            )
        )


def check_route(url: str) -> None:
    # use to check which route is used for defined url, example:
    # $ python -m eportal.cli.print_routes eportal/patches.json
    # ('get_kernel_file', {'kernel': 'eportal', 'fname': 'patches.json'})
    res = app.url_map.bind(server_name='garbage').match(path_info=url)
    print(res)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        check_route(sys.argv[1])
    else:
        print_routes(app)
