import sys
import gc
import pdb
import collections
from typing import Dict, Set, Union, Type, Tuple
from typing import Counter as CounterT
from typing_extensions import TypeAlias

from eportal.web import app

_refs: Set[int] = set()

KeyT: TypeAlias = Union[str, Type]


def get_mem_stats() -> Tuple[CounterT[KeyT], CounterT[KeyT], int, Dict[KeyT, list]]:
    counts: CounterT[KeyT] = collections.Counter()
    example: Dict[KeyT, list] = {}
    sizes: CounterT[KeyT] = collections.Counter()
    total = 0
    key: KeyT
    for o in gc.get_objects():
        if id(o) in _refs:
            continue
        if type(o) is dict:
            if 'server_id' in o:
                key = 'checkin_dict'
            else:
                rf = gc.get_referrers(o)
                if type(rf[0]) is not type and type(type(rf[0])) is type:
                    key = str(type(rf[0])) + '_dict'
                else:
                    key = type(o)

        elif type(o) is type:
            key = o
        else:
            key = 'instance of ' + str(type(o))

        e = example.setdefault(key, [])
        if len(e) < 10:
            e.append(o)
        counts[key] += 1
        sz = sys.getsizeof(o, 0)
        sizes[key] += sz
        total += sz
    return sizes, counts, total, example


def dump_mem_stats(filename: str) -> None:
    sizes, counts, total, example = get_mem_stats()
    with open(filename, 'w') as f:
        print('Total', total, 1, sep='\t', file=f)
        for k, v in sizes.most_common(100):
            print(str(k), v, counts[k], example[k], sep='\t', file=f)
            print('=======', file=f)


@app.route('/test/mem-snapshot')
def get_memory_snapshot() -> str:
    _refs.clear()
    _refs.add(id(_refs))
    for o in gc.get_objects():
        _refs.add(id(o))
    return 'Done\n'


@app.route('/test/set-trace')
def set_trace() -> str:
    pdb.set_trace()
    return 'Done\n'


@app.route('/test/dump-mem-stats')
def get_memory_stats() -> str:
    # Ignore use of hardcoded file path in /tmp, it is expected as part of this API call.
    fname = '/tmp/memory.txt'  # nosec B108
    dump_mem_stats(fname)
    return f'Memory report is in {fname}\n'
