from datetime import datetime

from sqlalchemy.orm.decl_api import DeclarativeBase
from sqlalchemy import String, Integer, Boolean, DateTime, Index
from sqlalchemy.orm.decl_api import Mapped
from sqlalchemy.orm.decl_api import mapped_column


class Base(DeclarativeBase):
    pass


class Patch(Base):
    __tablename__ = 'patches'
    release: Mapped[str] = mapped_column(String, nullable=False, primary_key=True)
    kernel_id: Mapped[str] = mapped_column(String, nullable=False, primary_key=True)
    patch_level: Mapped[int] = mapped_column(Integer, nullable=False, primary_key=True)
    version: Mapped[int] = mapped_column(Integer, nullable=False)


Index('idx_patches_kernel_id_patch_level', Patch.kernel_id, Patch.patch_level.desc())


class Fixup(Base):
    __tablename__ = 'fixups'
    release: Mapped[str] = mapped_column(String, nullable=False, primary_key=True)
    kernel_id: Mapped[str] = mapped_column(String, nullable=False, primary_key=True)
    fixup_for: Mapped[str] = mapped_column(Integer, nullable=False, primary_key=True)
    fixup_from: Mapped[int] = mapped_column(Integer, nullable=False)


Index('idx_fixups_kernel_for_from', Fixup.kernel_id, Fixup.fixup_for, Fixup.fixup_from)


class Release(Base):
    __tablename__ = 'releases'
    unit_id: Mapped[int] = mapped_column(Integer, nullable=False, default=0, server_default='0', primary_key=True)
    stream: Mapped[str] = mapped_column(String, nullable=False, primary_key=True)
    release: Mapped[str] = mapped_column(String, nullable=False, primary_key=True)
    feed: Mapped[str] = mapped_column(String, nullable=False, primary_key=True)
    deployed: Mapped[datetime] = mapped_column(DateTime, nullable=False)
    enabled: Mapped[bool] = mapped_column(Boolean, nullable=False)


Index('idx_releases_stream_release', Release.stream, Release.release)
