import dataclasses

from eportal import utils


class Product(utils.EnumEx):
    KERNEL = 'kernel', 'KernelCare'
    LIBCARE = 'libcare', 'LibCare'
    QEMU = 'qemu', 'QemuCare'
    DB = 'db', 'DBCare'
    TUXCARE_ESU = 'tuxcare-esu', 'Enterprise Support for AlmaLinux - ESU'


class Stream(utils.EnumEx):
    product: Product
    psvalue: str

    KERNEL = 'kernel', 'KernelCare', Product.KERNEL, 'kernel'
    USER = 'user', 'LibCare', Product.LIBCARE, 'libcare'
    QEMU = 'qemu', 'QEMUCare', Product.QEMU, 'qemu'
    DB = 'db', 'DBCare', Product.DB, 'db'


class Channel(utils.EnumEx):
    psvalue: str  # actual patchserver channel value

    DEFAULT = 'default', 'Stable', 'stable'
    TEST = 'test', 'Testing', 'test'
    UNSTABLE = 'unstable', 'Unstable', 'unstable'


class PSProduct(utils.EnumEx):
    psvalue: str  # actual patchserver product value

    KCE = 'default', 'KCE', ''
    _ALMACARE_CS = 'almacare-cs', 'AlmaCare Cybersecurity', 'almacare-cs'  # deprecated


@dataclasses.dataclass(frozen=True)
class PSFetchSpec:
    psproduct: PSProduct
    channel: Channel


DEFAULT_PRODUCTS = [it.value for it in (Product.KERNEL, Product.LIBCARE, Product.QEMU, Product.DB)]
PRODUCTS_INDUCING_SERVER_CHECKINS = [
    Product.KERNEL,
]
USER_STREAMS = Stream.USER, Stream.QEMU, Stream.DB
REPLICATED_STREAMS = list(Stream)

USER_PREFIX_STREAM_MAP = {
    'u': Stream.USER,
    'db': Stream.DB,
    'qemu': Stream.QEMU,
}
