from typing import Union, Optional

import config
from eportal import patches_constants as pc
from eportal.patches.serve import PatchStore, CachedPatchStore
from eportal.deploy import PatchDeployManager
from eportal.patchsource import PatchSource


def get_repo(
    unit_id: int,
    stream: pc.Stream,
    patchsource: Optional[PatchSource] = None,
    is_local: Optional[bool] = None,
    forbidden_ok: bool = True,
) -> PatchDeployManager:
    return PatchDeployManager(get_store(unit_id, stream), patchsource, is_local=is_local, forbidden_ok=forbidden_ok)


def get_store(unit_id: int, stream: pc.Stream) -> Union[CachedPatchStore, PatchStore]:
    if stream == pc.Stream.KERNEL:
        if config.CACHE_MODE:
            return CachedPatchStore(unit_id, stream, config.PATCH_RESOURCES_DIR, config.ARCH_DIR)
        else:
            return PatchStore(unit_id, stream, config.PATCH_RESOURCES_DIR, config.ARCH_DIR)
    elif stream == pc.Stream.USER:
        return PatchStore(unit_id, stream, config.USER_PATCH_RESOURCES_DIR, config.ARCH_DIR)
    elif stream == pc.Stream.QEMU:
        return PatchStore(unit_id, stream, config.USER_PATCH_RESOURCES_DIR, config.ARCH_DIR)
    elif stream == pc.Stream.DB:
        return PatchStore(unit_id, stream, config.USER_PATCH_RESOURCES_DIR, config.ARCH_DIR)
    else:  # pragma: no cover
        raise ValueError(f"Invalid store type {stream!r}")
