from typing import Set, cast, Optional, Tuple, Union, Any

from flask_login import current_user

from eportal import utils

SUPERADMIN = 'superadmin'
NONE = 'none'

DEPLOY = 'deploy'
MANAGE_KEYS = 'manage-keys'
MANAGE_FEEDS = 'manage-feeds'
MANAGE_USERS = 'manage-users'
MANAGE_CONFIG = 'manage-config'
MANAGE_SERVERS = 'manage-servers'
EXPORT_DATA = 'export-data'
VIEW = 'view'
AUDIT = 'audit'

MANAGE_UNIT = 'manage-unit'
MANAGE_UNIT_USERS = 'manage-unit-users'
EDIT_UNIT_TITLE = 'edit-unit-title'
VIEW_UNIT = 'view-unit'

Perm = Union[str, Tuple[str, Any]]
PermSet = Set[Perm]


class Roles(utils.EnumEx):
    permissions: PermSet

    INACTIVE = 'inactive', 'Inactive', cast(PermSet, set())
    ADMIN = 'admin', 'Administrator', {SUPERADMIN}
    VIEWER = 'viewer', 'Viewer', {VIEW}


class UnitRoles(utils.EnumEx):
    permissions: PermSet

    INACTIVE = 'inactive', 'Inactive', cast(PermSet, set())
    ADMIN = (
        'admin',
        'Unit Administrator',
        {DEPLOY, MANAGE_KEYS, MANAGE_FEEDS, MANAGE_UNIT_USERS, MANAGE_SERVERS, AUDIT, VIEW, EDIT_UNIT_TITLE, VIEW_UNIT},
    )
    MANAGER = 'patch-manager', 'Unit Patch Manager', {DEPLOY, MANAGE_KEYS, MANAGE_SERVERS, VIEW}
    VIEWER = 'viewer', 'Viewer', {VIEW}

    @staticmethod
    def get(value: Optional[str]) -> 'UnitRoles':
        return UnitRoles.items.get(value, UnitRoles.INACTIVE)  # type: ignore[arg-type]


def to_permset(perm: Union[Perm, PermSet]) -> PermSet:
    if isinstance(perm, set):
        return perm
    return {perm}


def has_permission(user_permissions: PermSet, permission: Perm) -> bool:
    return SUPERADMIN in user_permissions or permission in user_permissions


def has_permissions(user_permissions: PermSet, permissions: PermSet) -> bool:
    return SUPERADMIN in user_permissions or bool(permissions & user_permissions)


def user_has_permission(permission: Perm) -> bool:
    return has_permission(current_user.permissions, permission)
