from functools import wraps
from typing import TypeVar, Callable, cast

from flask import g

R = TypeVar('R', covariant=True)


def g_cache(fn: Callable[[], R]) -> Callable[[], R]:
    @wraps(fn)
    def inner() -> R:
        try:
            return cast(R, getattr(g, fn.__name__))
        except AttributeError:
            pass

        result = fn()
        setattr(g, fn.__name__, result)
        return result

    return inner
