from logging import getLogger
from typing import List

from eportal import patches_constants as pc
from eportal.db.models import KcServer, DEFAULT_FEED, get_key
from eportal.patches import api
from eportal.web import app

logger = getLogger(__name__)


def nagios_header(list_: List[KcServer], title: str) -> str:
    count = len(list_)
    if count == 1:
        result = "1 machine is "
    else:
        result = str(count) + " machines are "
    return result + title + "!|"


def nagios_entries(list_: List[KcServer], title: str) -> str:
    result = ""
    for server in list_:
        result += "%s (%s) is %s\n" % (server.node, server.remote_ip, title)
    return result


@app.route('/admin/api/kcare/nagios/<string:key>')
def nagios_check(key: str) -> str:
    kc_key = get_key(key)
    if kc_key:
        servers = kc_key.servers
        uptodate = 0
        outofdate = []
        unsupported = []
        inactive = []
        from datetime import datetime, timedelta

        checkin_window = datetime.now() - timedelta(hours=12)
        inactive_window = checkin_window - timedelta(hours=36)

        feed = kc_key.feed
        if feed is None:
            feed = DEFAULT_FEED

        kernel_to_latest = dict(api.get_kernel_level_pairs(kc_key.unit_id, pc.Stream.KERNEL, feed))

        result = ""
        for server in servers:
            if server.kernel_id in kernel_to_latest:
                latest = kernel_to_latest[server.kernel_id]

                if server.patch_level is not None and latest <= server.patch_level:  # running latest patch_level
                    uptodate += 1
                # old patch level + hasn't checked in for 2 days
                elif server.last is not None and server.last < inactive_window:
                    inactive.append(server)
                # old patch level + hasn't checked in for 12 hours
                elif server.last is not None and server.last < checkin_window:
                    outofdate.append(server)
                # did checkin in the past 12 hours, and patch_level + 1 == current
                elif server.patch_level is not None and server.patch_level + 1 == latest:
                    uptodate += 1
                else:  # old patch_level & did checkin in 12 hours
                    outofdate.append(server)
            else:
                unsupported.append(server)

        if outofdate:
            result += nagios_header(outofdate, "OUTOFDATE")
        elif inactive:
            result += nagios_header(inactive, "INACTIVE")
        elif unsupported:
            result += nagios_header(unsupported, "UNSUPPORTED")
        result += "uptodate=%d;outofdate=%d;unsupported=%d;inactive=%d\n" % (
            uptodate,
            len(outofdate),
            len(unsupported),
            len(inactive),
        )
        result += nagios_entries(outofdate, "OUTOFDATE")
        result += nagios_entries(inactive, "INACTIVE")
        result += nagios_entries(unsupported, "UNSUPPORTED")

        return result

    else:
        message = 'Registration token not found'
        logger.error(message)
        return '{0}\n'.format(message)
