import sys

sys.path.insert(0, '.')

from alembic import context
from logging.config import fileConfig
from sqlalchemy.engine.reflection import Inspector

config = context.config
if config.config_file_name is not None:
    fileConfig(config.config_file_name)

if config.config_ini_section == 'patches-data':
    from eportal import patches as db
elif config.config_ini_section == 'spooler':
    from eportal import spooler as db  # type: ignore[no-redef]
else:
    from eportal import db  # type: ignore[no-redef]

if hasattr(db, 'import_all'):
    db.import_all()
target_metadata = db.Base.metadata


def run_migrations_offline():
    context.configure(url=db.engine.url.render_as_string(), target_metadata=target_metadata, literal_binds=True)
    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    # restrict automigrations to defined models only
    # note: in case of table deletion you need to delete table manually
    get_table_names = Inspector.get_table_names
    Inspector.get_table_names = lambda *args, **kwargs: list(  # type: ignore[method-assign]
        set(get_table_names(*args, **kwargs)).intersection(target_metadata.tables)
    )

    with db.engine.connect() as connection:
        context.configure(connection=connection, target_metadata=target_metadata)
        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()
