"""add unit_id into all models

Revision ID: 487a98017b0a
Revises: c58846b59235
Create Date: 2024-03-01 15:53:33.943881

"""

from contextlib import contextmanager

# revision identifiers, used by Alembic.
revision = '487a98017b0a'
down_revision = 'c58846b59235'
branch_labels = None

from alembic import op
import sqlalchemy as sa


@contextmanager
def change_pk(table, flist):
    tnew = table + '_new'
    yield tnew
    op.execute(f'INSERT INTO {tnew} ({flist}) SELECT {flist} FROM {table}')
    op.drop_table(table)
    op.rename_table(tnew, table)


def upgrade():
    op.add_column('auditlog', sa.Column('unit_id', sa.Integer(), server_default='0', nullable=False))
    op.add_column('kc_keys', sa.Column('unit_id', sa.Integer(), server_default='0', nullable=False))

    # alembic doesn't render valid PK for batch op, so here is a manual migration
    with change_pk('feed', 'name, auto, deploy_after, channel') as tmp:
        op.create_table(
            tmp,
            sa.Column('unit_id', sa.Integer(), server_default='0', nullable=False),
            sa.Column('name', sa.Text(), nullable=False),
            sa.Column('auto', sa.Boolean(), nullable=True),
            sa.Column('deploy_after', sa.Integer(), nullable=True),
            sa.Column('channel', sa.Text(), server_default='default', nullable=True),
            sa.PrimaryKeyConstraint('unit_id', 'name'),
        )

    with change_pk('tag', 'name') as tmp:
        op.create_table(
            tmp,
            sa.Column('unit_id', sa.Integer(), server_default='0', nullable=False),
            sa.Column('name', sa.String(), nullable=False),
            sa.PrimaryKeyConstraint('unit_id', 'name'),
        )


def downgrade():
    op.drop_column('kc_keys', 'unit_id')
    op.drop_column('auditlog', 'unit_id')

    # at least for drop column it works fine
    op.execute('DELETE FROM feed WHERE unit_id != 0')
    with op.batch_alter_table('feed', table_args=(sa.PrimaryKeyConstraint('name'),)) as bop:
        bop.drop_column('unit_id')

    op.execute('DELETE FROM tag WHERE unit_id != 0')
    with op.batch_alter_table('tag', table_args=(sa.PrimaryKeyConstraint('name'),)) as bop:
        bop.drop_column('unit_id')
