var app = new Vue({
    el: '#app',
    data: {
        distros: getDistroChoices(distro_map),
        feeds: [],
        patches: {},
        distro_map: distro_map,
        isLoading: false
    },
    mounted() {
        this.loadFeeds();
        this.loadPatches();
    },
    computed: {
        sorted_patches: function () {
            var items = _.pairs(this.patches)
            if (this.$route.query.distro) {
                var flavors = this.$route.query.distro.split(',')
                items = _.filter(items, patch => ~flavors.indexOf(patch[1].flavor || patch[1].distro))
            }
            if (this.$route.query.search) {
                items = _.filter(items, item => item[1].name.includes(this.$route.query.search))
            }
            return _.sortBy(items, item => item[1].timestamp).reverse()
        }
    },
    watch: {
        patches: function (new_patches, old_patches) {
            this.distro_map = {}
            for (var item in new_patches) {
                var flavor = new_patches[item].flavor || new_patches[item].distro
                this.distro_map[flavor] = distro_map[flavor] || flavor
            }
            this.distros = getDistroChoices(this.distro_map)
        }
    },
    methods: {
        navURL: function (name, value) {
            var query = Object.assign({}, this.$route.query)
            query[name] = value
            return '/?' + 'type=' + (query.type || '') + '&' + 'feed=' + (query.feed || '')
        },
        infoURL: function (hash, latest) {
            if (this.$route.query.type === 'libcare') {
                return '/u/' + hash + '/' + latest + '/info.html'
            } else if (this.$route.query.type === 'qemu') {
                return '/qemu/' + hash + '/' + latest + '/info.html'
            } else if (this.$route.query.type === 'db') {
                return '/db/' + hash + '/' + latest + '/info.html'
            } else {
                if (this.$route.query.feed || '' != '') {
                    return '/' + this.$route.query.feed + '/' + hash + '/' + latest + '/kpatch.html'
                } else {
                    return '/' + hash + '/' + latest + '/kpatch.html'
                }
            }
        },
        updateURL: function (name, value) {
            var query = Object.assign({}, this.$route.query)
            query[name] = value
            this.$router.push({path: '/', query: query})
        },
        updateSearch: _.debounce(function (event) {
            this.updateURL('search', event.target.value)
        }, 500),
        updateDistro: function (event) {
            this.updateURL('distro', event.target.value)
        },
        loadFeeds: function () {
            fetch('/admin/api/feed-names')
                .then(response => response.json())
                .then(json => this.feeds = json.result);
        },
        loadPatches: function () {
            var patches_json = '/patches.json'
            if (this.$route.query.type === 'libcare') {
                patches_json = '/userspace-patches.json'
            } else if (this.$route.query.type === 'qemu') {
                patches_json = '/qemu-patches.json'
            } else if (this.$route.query.type === 'db') {
                patches_json = '/db-patches.json'
            }

            var feed_path = ''
            if (this.$route.query.feed) {
                feed_path = '/' + this.$route.query.feed
            }

            this.isLoading = true;
            fetch(feed_path + patches_json)
                .then(response => response.json())
                .then(json => this.patches = json)
                .finally(() => this.isLoading = false);
        },
    },
    router: new VueRouter({
        routes: [{path: '/'}],
        mode: 'history',
    })
})